#include "X3D_X3DGroupingNode.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

X3DGroupingNode::X3DGroupingNode()
: X3DChildNode()
{
  defineTypeName("X3DGroupingNode", "Grouping");
}

X3DGroupingNode::X3DGroupingNode(const X3DGroupingNode &N)
: X3DChildNode(N)
{
  MFAbstractNode childrenList = N.getChildList();
  for (MFAbstractNode::const_iterator it = childrenList.begin(); it != childrenList.end(); ++it)
    setChild(*it);
}

X3DGroupingNode::~X3DGroupingNode()
{
  for (MFNode::iterator it = _children.begin(); it != _children.end(); ++it)
    removeParentFromChild(this, *it);
}

MFAbstractNode X3DGroupingNode::getChildList() const
{
  MFAbstractNode childrenList;
  for (MFNode::const_iterator it = _children.begin(); it != _children.end(); ++it)
    if (*it != 0)
      childrenList.push_back(*it);
    
  return childrenList; 
}

bool X3DGroupingNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    addParentToChild(this, N);
    return true;
  }
  if (N != 0)
    cx3d << "X3D::X3DGroupingNode::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::X3DGroupingNode::addChild : cannot add a NULL node!" << endl;
      
  
  return false;    
}

bool X3DGroupingNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    addParentToChild(this, N);
    return true;
  }
  if (N != 0)
    cx3d << "X3D::X3DGroupingNode::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::X3DGroupingNode::setChild : cannot add a NULL node!" << endl;
  
  return false;    
}

bool X3DGroupingNode::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_children.begin(), _children.end(), N);
  if (res != _children.end())
  {
    removeParentFromChild(this, N);
    _children.erase(res);
    return true;
  }  
    
  return false;
}

}
}
