#include "X3D_SpotLight.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SpotLight::SpotLight()
  : X3DLightNode(),
    _attenuation(SFVec3f(1.0f,0.0f,0.0f)),
    _beamWidth  (1.570796f),
    _cutOffAngle(0.785398f),
    _direction  (SFVec3f(0.0f,0.0f,-1.0f)),
    _location   (SFVec3f(0.0f,0.0f,0.0f)),
    _radius     (100.0f)
{
  defineTypeName("SpotLight", "Lighting");
}

SpotLight::SpotLight(const SFFloat       &ambientIntensity,
                     const SFVec3f &attenuation,
                     const SFFloat       &beamWidth,
                     const SFColor &color,
                     const SFFloat       &cutOffAngle,
                     const SFVec3f &direction,
                     const SFFloat       &intensity,
                     const SFVec3f &location,
                     const SFBool        &on,
                     const SFFloat       &radius)
  : X3DLightNode(ambientIntensity,color,intensity,on),
    _attenuation(attenuation),
    _beamWidth  (beamWidth),
    _cutOffAngle(cutOffAngle),
    _direction  (direction),
    _location   (location),
    _radius     (radius)
{
  defineTypeName("SpotLight", "Lighting");
}

SpotLight::SpotLight(const SpotLight &N)
  : X3DLightNode(N),
    _attenuation(N._attenuation),
    _beamWidth  (N._beamWidth),
    _cutOffAngle(N._cutOffAngle),
    _direction  (N._direction),
    _location   (N._location),
    _radius     (N._radius)
{
}

SFAbstractNode SpotLight::clone() const
{
  return new SpotLight(*this);
}

SpotLight::~SpotLight()
{
}

void SpotLight::setAttenuation(const SFVec3f &attenuation)
{
  _attenuation = attenuation;
}

void SpotLight::setBeamWidth(const SFFloat &beamWidth)
{
  _beamWidth = beamWidth;
}

void SpotLight::setCutOffAngle(const SFFloat &cutOffAngle)
{
  _cutOffAngle = cutOffAngle;
}

void SpotLight::setDirection(const SFVec3f &direction)
{
  _direction = direction;
}

void SpotLight::setLocation(const SFVec3f &location)
{
  _location = location;
}

void SpotLight::setRadius(const SFFloat &radius)
{
  _radius = radius;
}

void SpotLight::load(const X3DFileElement *element)
{
  X3DLightNode::load(element);

  int index;
  index = element->getIndexAttribute("attenuation");
  if (index != -1)
    _attenuation = SFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("beamWidth");
  if (index != -1)
    _beamWidth = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("cutOffAngle");
  if (index != -1)
    _cutOffAngle = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("direction");
  if (index != -1)
    _direction = SFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("location");
  if (index != -1)
    _location = SFVec3f(element->getAttribute(index));

  index = element->getIndexAttribute("radius");
  if (index != -1)
    _radius = element->getAttribute(index).toFloat();
}

SFString &SpotLight::write(SFString &output) const
{
  X3DLightNode::write(output);
  
  if (_attenuation != SFVec3f(1.0f,0.0f,0.0f))  
    output += " attenuation=\"" + toSFString(_attenuation) + "\"";
  if (_beamWidth != 1.570796f)  
    output += " beamWidth=\"" + toSFString(_beamWidth) + "\"";
  if (_cutOffAngle != 0.785398f)  
    output += " cutOffAngle=\"" + toSFString(_cutOffAngle) + "\"";
  if (_direction != SFVec3f(0.0f,0.0f,-1.0f))  
    output += " direction=\"" + toSFString(_direction) + "\"";
  if (_location != SFVec3f(0.0f,0.0f,0.0f))  
    output += " location=\"" + toSFString(_location) + "\"";
  if (_radius != 100.0f)  
    output += " radius=\"" + toSFString(_radius) + "\"";
    
  return output;  
}

}
}
