#include "X3D_TriangleFanSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TriangleFanSet::TriangleFanSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("TriangleFanSet", "Rendering");
}

TriangleFanSet::TriangleFanSet(X3DColorNode *color, X3DCoordinateNode *coord, X3DNormalNode *normal, X3DTextureCoordinateNode *texCoord, const SFBool &ccw, const SFBool &colorPerVertex, const SFBool &convex, const SFFloat &creaseAngle, const MFInt32 &fanCount, const SFBool &normalPerVertex, const SFBool &solid)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_convex(convex), _creaseAngle(creaseAngle), _fanCount(fanCount)
{
  defineTypeName("TriangleFanSet", "Rendering");
}
           
TriangleFanSet::TriangleFanSet(const TriangleFanSet &T)
: X3DComposedGeometryNode(T), _convex(T._convex), _creaseAngle(T._creaseAngle), _fanCount(T._fanCount)
{
}

SFAbstractNode TriangleFanSet::clone() const
{
  return new TriangleFanSet(*this);
}

TriangleFanSet::~TriangleFanSet()
{
}

void TriangleFanSet::setConvex(const SFBool &convex)
{
  _convex = convex;
}

void TriangleFanSet::setCreaseAngle(const SFFloat &creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void TriangleFanSet::setFanCount(const MFInt32 &fanCount)
{
  _fanCount  = fanCount;
}

void TriangleFanSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);
  
  int index;
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("fanCount");
  if (index != -1)
    _fanCount = MFInt32(element->getAttribute(index));  
}

SFString &TriangleFanSet::write(SFString &output) const
{
  X3DComposedGeometryNode::write(output);
  
  if (!_convex)  
    output += " convex=\"FALSE\"";
  if (_creaseAngle != 0.0f)
    output += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
  if (!_fanCount.empty())
    output += " fanCount=\"" + toSFString(_fanCount) + "\"";
  
  return output;
}

}
}
