#include "X3D_FillProperties.h"

namespace X3DTK {
namespace X3D {

FillProperties::FillProperties()
: X3DAppearanceChildNode(), _fillStyle(SFString("")), _hatchStyle(1), _hatchColor(SFColor(1.0f, 1.0f, 1.0f))
{
  defineTypeName("FillProperties", "Shape");
}

FillProperties::FillProperties(const SFString &fillStyle, const SFInt32 &hatchStyle, const SFColor &hatchColor)
: X3DAppearanceChildNode(), _fillStyle(fillStyle), _hatchStyle(hatchStyle), _hatchColor(hatchColor)
{
  defineTypeName("FillProperties", "Shape");
}

FillProperties::FillProperties(const FillProperties &F)
: X3DAppearanceChildNode(F), _fillStyle(F._fillStyle), _hatchStyle(F._hatchStyle), _hatchColor(F._hatchColor)
{
}

SFAbstractNode FillProperties::clone() const
{
  return new FillProperties(*this);
}

FillProperties::~FillProperties()
{
}

void FillProperties::setFillStyle(const SFString &fillStyle)
{
  _fillStyle = fillStyle;
}

void FillProperties::setHatchStyle(const SFInt32 &hatchStyle)
{
  _hatchStyle = hatchStyle;
}

void FillProperties::setHatchColor(const SFColor &hatchColor)
{
  _hatchColor = hatchColor;
}

void FillProperties::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("fillStyle");
  if (index != -1)
    _fillStyle = element->getAttribute(index);
  
  index = element->getIndexAttribute("hatchStyle");
  if (index != -1)
    _hatchStyle = element->getAttribute(index).toInt();  
  
  index = element->getIndexAttribute("hatchColor");
  if (index != -1)
    _hatchColor = SFColor(element->getAttribute(index));    
}

SFString &FillProperties::write(SFString &output) const
{
  if (_fillStyle != "NONE")
    output += " fillStyle = \"" + _fillStyle + "\"";
  if (_hatchStyle != 1)
    output += " hatchStyle = \"" + toSFString(_hatchStyle) + "\"";  
  if (_hatchColor != SFColor(1.0f, 1.0f, 1.0f))
    output += " hatchColor = \"" + toSFString(_hatchColor) + "\"";
      
  return output;
}

}
}
