#include "X3D_GLBuilder.h"
#include "DFSGraphTraversal.h"
#include "X3D_GLBuilderCoreVisitor.h"
#include "X3D_GLBuilderGeometry3DVisitor.h"
#include "X3D_GLBuilderGroupingVisitor.h"
#include "X3D_GLBuilderRenderingVisitor.h"
#include "X3D_GLBuilderShapeVisitor.h"
#include "X3D_GLBuilderTexturingVisitor.h"
#include "X3D_GLBuilderNavigationVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilder::GLBuilder()
: X3DGLBuilder()
{
  stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
  graphTraversal = new DFSGraphTraversal();
  
  graphTraversal->setComponentVisitor(new GLBuilderCoreVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderGeometry3DVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderGroupingVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderRenderingVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderShapeVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderTexturingVisitor());
  graphTraversal->setComponentVisitor(new GLBuilderNavigationVisitor());
}

GLBuilder::~GLBuilder()
{
  GraphTraversal::removeInstanceOf<GLBuilderStateVariables>();
  
  delete graphTraversal;
}

GL::SFNode GLBuilder::build(SFNode N)
{
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish();
    
  return stateVariables->getRoot();
}

GL::Scene *GLBuilder::build(Scene *S)
{
  return static_cast<GL::Scene *>(build(static_cast<SFNode>(S)));
}

}
}
